#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <string.h>
#include <string>
#include <libosso.h>
#include <gtk/gtk.h>

#include "dbus-print-message.h"

using namespace std;

// gosmore forwards
void maemoTimerTick(void);

// local forwards
void setupDBUSConnection(void);

static	osso_context_t* osso_ctx = NULL;
GtkWidget * splashWindow = NULL;

void initMaemo(void) {

  /* LibOSSO context object/connection */
  printf("Initializing LibOSSO context (org.openstreetmap.gosmore)\n");
  osso_ctx = osso_initialize("org.openstreetmap.gosmore", "svn20100214", TRUE, NULL);
  if(osso_ctx == NULL) {
    printf("Failed to init LibOSSO\n");
    return;
  }
  printf("LibOSSO Init done\n");

  setupDBUSConnection();
}

void cleanupMaemo(void) {
	if(osso_ctx != NULL) 
		osso_deinitialize(osso_ctx);
}

#ifdef HAVE_GPSBT
#include <gpsbt.h>
static gpsbt_t gpsbt_context;
static int gpsbtStarted = 0;

void startMaemoGPS(void) {
	char errstr[256] = "";
	// start gpsd (via gpsbt) first
	errno = 0;
	memset(&gpsbt_context, 0, sizeof(gpsbt_t));
	if(gpsbt_start(NULL, 0, 0, 0, errstr, sizeof(errstr), 0, &gpsbt_context) < 0) {
		printf("Error connecting to GPS with gpsbt: (%d) %s (%s)\n", errno, strerror(errno), errstr);
	} else
    gpsbtStarted = 1;
	sleep(1);       /* give gpsd time to start */
	printf("gpsbt_start: completed\n");
}

void stopMaemoGPS(void) {
  if(!gpsbtStarted)
    return;

	int err = gpsbt_stop(&gpsbt_context);
	if(err < 0) {
		printf("Error %d while gpsbt_stop", err);
	} else
    gpsbtStarted = 0;
	printf("gpsbt_stop: completed, (%d)",err);
}

void showSplashScreen(void) {
  GtkWidget  *image;
  GtkWidget  *label;
  string splashImageFile;

  splashWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_type_hint(GTK_WINDOW(splashWindow), GDK_WINDOW_TYPE_HINT_SPLASHSCREEN);

  splashImageFile = (string) RES_DIR + "splash.png";
  image = gtk_image_new_from_file(splashImageFile.c_str());
  gtk_container_add (GTK_CONTAINER (splashWindow), image);

  // label = gtk_label_new ("Gosmore");
  // gtk_container_add (GTK_CONTAINER (splashWindow), label);
  // gtk_widget_show_all(label);

  gtk_widget_show_all(splashWindow);
  
  while(gtk_events_pending())
    gtk_main_iteration();
}

void hideSplashScreen(void) {
  if(splashWindow != NULL)
    gtk_widget_hide(splashWindow);
}
 
gint gpsDataHandler = 0;
gboolean useNextGPSData = FALSE;

gboolean handleThisGPSData(void) {
  if(useNextGPSData) {
    useNextGPSData = FALSE;
    return TRUE;
  } else
    return FALSE;
}

gboolean handleGPSDataTimer(gpointer data) {
  useNextGPSData = TRUE;

  // hacked in here
  maemoTimerTick();

  return TRUE;
}

void installGPSDataHandler(void) {
  gpsDataHandler = gtk_timeout_add(5000, handleGPSDataTimer, NULL);
}

void uninstallGPSDataHandler(void) {
  gtk_timeout_remove(gpsDataHandler);
}

void suppressScreenBlanking(void) {
	if(osso_ctx == NULL) 
    return;
  osso_display_state_on(osso_ctx);
  osso_display_blanking_pause(osso_ctx);
}

DBusHandlerResult
dbus_filter_func(DBusConnection     *connection,
                 DBusMessage        *message,
                 void               *user_data) {
  // print_dbus_message(message, FALSE);  
  return DBUS_HANDLER_RESULT_HANDLED;
}

void setupDBUSConnection(void) {
  /* low level dbus */
  DBusConnection *dbusCon;
  DBusError error;
  DBusBusType type = DBUS_BUS_SYSTEM; //DBUS_BUS_SESSION;
  dbus_error_init (&error);
  dbusCon = dbus_bus_get (type, &error);
  if(dbusCon == NULL) {
    fprintf (stderr, "Failed to open connection to %s message bus: %s\n",
	      (type == DBUS_BUS_SYSTEM) ? "system" : "session",
              error.message);
    dbus_error_free (&error);
  }

  dbus_bus_add_match(dbusCon, "type='signal'", &error);
  // dbus_bus_add_match(dbusCon, "type='signal',interface='com.nokia.mce.signal'", &error);
  if(dbus_error_is_set (&error))
    fprintf (stderr, "Error: %s\n", error.message);

  if (!dbus_connection_add_filter(dbusCon, dbus_filter_func, NULL, NULL)) 
    fprintf (stderr, "Couldn't add dbus filter!\n");

  printf("DBus setup done\n");
}

#endif
